<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Plugins\Model\QuoteRepository;

use Exception;
use Magento\Directory\Helper\Data;
use Magento\Directory\Model\Region;
use Magento\Directory\Model\ResourceModel\Region\CollectionFactory;
use Magento\Quote\Api\Data\AddressInterfaceFactory;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Model\Quote\Address\Rate;

/**
 * @see \Magento\Quote\Model\QuoteRepository\LoadHandler
 */
class LoadHandler
{

	/**
	 * @var AddressInterfaceFactory
	 */
	private $addressFactory;


	/**
	 * @var Data
	 */
	private $data;
	/**
	 * @var CollectionFactory
	 */
	private $collectionFactory;

	/**
	 * @param AddressInterfaceFactory $addressFactory
	 * @param Data $data
	 * @param CollectionFactory $collectionFactory
	 */
	public function __construct(AddressInterfaceFactory $addressFactory,
	                            Data                    $data,
	                            CollectionFactory       $collectionFactory
	)
	{
		$this->addressFactory = $addressFactory;
		$this->data = $data;
		$this->collectionFactory = $collectionFactory;
	}

	/**
	 * @throws Exception
	 * @see \Magento\Quote\Model\QuoteRepository\LoadHandler::load
	 *
	 */
	public function afterLoad(\Magento\Quote\Model\QuoteRepository\LoadHandler $subject, CartInterface $quote)
	{
		if (!$quote->getIsActive()) {
			return $quote;
		}

		$shippingAddress = $quote->getShippingAddress();

		if (!$shippingAddress) {
			$shippingAddress = $this->addressFactory->create();
		}

		if (!$quote->hasItems()) {
			return $quote;
		}

		if (!$shippingAddress->getCountryId() || !$shippingAddress->getShippingMethod()) {

			$defaultCountry = $this->data->getDefaultCountry($quote->getStoreId());
			$shippingAddress->setCountryId($defaultCountry);

			if ($this->data->isRegionRequired($defaultCountry)) {
				/** @var Region $region */
				$region = $this->collectionFactory->create()
					->addCountryFilter($defaultCountry)
					->getFirstItem();
				$shippingAddress->setRegion($region->getName());
			}

			$shippingAddress
				->setCollectShippingRates(true)
				->collectShippingRates();
			$shippingMethods = $shippingAddress->getAllShippingRates();
			/** @var Rate $fist */
			$fist = reset($shippingMethods);
			$shippingAddress->setShippingMethod($fist->getCode());
			$quote->setShippingAddress($shippingAddress);
			$quote->collectTotals();
			$quote->save();
		}

		return $quote;
	}


}